#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include <Servo.h> 

#define MAX_SLOTS 5
#define LED_SLOT1 13
#define LED_SLOT2 12
#define LED_SLOT3 11
#define LED_SLOT4 10
#define LED_SLOT5 9
#define LED_SEM_GREEN 6 
#define LED_SEM_RED 7
#define IR_BEFORE 2
#define IR_AFTER 3
#define SM 5

Servo servo;
LiquidCrystal_I2C lcd(0x27, 16, 2);

int total_slots = 5;
int just_entered = 0;
int just_exited = 0;
int come = 0;
int leave = 0;
bool my_int0 = false;
bool my_int1 = false;


ISR(INT0_vect)
{ // Rutina de tratare a intreruperii externe pentru IR 1 (PD2)
  my_int0 = true;
}


ISR(INT1_vect)
{ // Rutina de tratare a intreruperii externe pentru IR 2 (PD3)
  my_int1 = true;
}


void setup_interrupts()
{
  cli();

  // Configurare IR1 1 (PD2) - intrerupere externa
  DDRD &= ~(1 << PD2);         // Seteaza pinul PD2 ca intrare
  PORTD |= (1 << PD2);

  EICRA |= (1 << ISC01);       // Configureaza intreruperea externa pe front descrescator (falling edge)
  EIMSK |= (1 << INT0);        // Activeaza intreruperea externa INT0

  // Configurare IR 2 (PD3) - intrerupere externa
  DDRD &= ~(1 << PD3);         // Seteaza pinul PD3 ca intrare
  PORTD |= (1 << PD3);

  EICRA |= (1 << ISC11);       // Configureaza intreruperea externa pe front descrescator (falling edge)
  EIMSK |= (1 << INT1);        // Activeaza intreruperea externa INT1

  sei();
}


void setup()
{
  setup_interrupts();

  DDRB |= (1 << PB5) | (1 << PB4) | (1 << PB3) | (1 << PB2) | (1 << PB1);
  DDRD |= (1 << PD6) | (1 << PD7);
  /* echivalentul cu functii:
  pinMode(LED_SLOT1, OUTPUT);
  pinMode(LED_SLOT2, OUTPUT);
  pinMode(LED_SLOT3, OUTPUT);
  pinMode(LED_SLOT4, OUTPUT);
  pinMode(LED_SLOT5, OUTPUT);
  pinMode(LED_SEM_GREEN, OUTPUT);
  pinMode(LED_SEM_RED, OUTPUT);
  */
 
  DDRD &= ~(1 << PD2) & ~(1 << PD3);
  /* echivalentul cu functii:
  pinMode(IR_BEFORE, INPUT);
  pinMode(IR_AFTER, INPUT);
  */

  PORTB |= (1 << PB5) | (1 << PB4) | (1 << PB3) | (1 << PB2) | (1 << PB1);
  PORTD |= (1 << PD6);
  PORTD &= ~(1 << PD7);
  /* echivalentul cu functii:
  digitalWrite(LED_SLOT1, HIGH);
  digitalWrite(LED_SLOT2, HIGH);
  digitalWrite(LED_SLOT3, HIGH);
  digitalWrite(LED_SLOT4, HIGH);
  digitalWrite(LED_SLOT5, HIGH);
  digitalWrite(LED_SEM_GREEN, HIGH);
  */

  servo.attach(SM);
  servo.write(110);

  lcd.init();
  lcd.backlight();
  lcd.setCursor(1, 0);
  lcd.print("Am PoliPark-at"); // mesajul initial
  lcd.setCursor(2, 1);
  lcd.print("Pret: 10 lei");
  delay(7000);
  lcd.clear();
}


void loop() {
  if (my_int0 == true && just_entered == 0) { // este masina la intrare
    if (total_slots > 0) {
      if (just_exited == 0) { // nu vine din interior
        servo.write(15);
        total_slots--;
        come = 1;
      }

      just_entered = 1;
      PORTD &= ~(1 << PD7); // digitalWrite(LED_SEM_RED, LOW);
      PORTD |= (1 << PD6); // digitalWrite(LED_SEM_GREEN, HIGH); -> semafor verde
    } else {
      PORTD |= (1 << PD7); // digitalWrite(LED_SEM_RED, HIGH); -> semafor rosu
      PORTD &= ~(1 << PD6); // digitalWrite(LED_SEM_GREEN, LOW);

      lcd.clear();
      lcd.setCursor(2, 0);
      lcd.print("Parcare plina!");
      lcd.setCursor(0, 1);
      lcd.print("Mai incearca!");
      delay(3000);
      lcd.clear();
      my_int0 = false;
    }
  }

  if (my_int1 == true && just_exited == 0) { // o masina pleaca
    if (total_slots != 0) { // este cel putin un loc liber
      PORTD &= ~(1 << PD7); // digitalWrite(LED_SEM_RED, LOW);
      PORTD |= (1 << PD6); // digitalWrite(LED_SEM_GREEN, HIGH); -> semafor verde
    }

    just_exited = 1;
    if (just_entered == 0) { // ridica bariera
      servo.write(15);
      total_slots++;
      leave = 1;
    }
  }

  if (just_entered == 1 && just_exited == 1) { // a trecut prin fata ambilor senzori
    if (leave == 1 && total_slots > 0) { // pleaca o masina
        lcd.setCursor(0, 0);
        lcd.print("Va mai asteptam!");
        lcd.setCursor(0, 1);
        lcd.print("                ");
      }
    
      if (come == 1 && total_slots < 5) { // intra o masina
        lcd.setCursor(1, 0);
        lcd.print("Parcheaza pe ");
        lcd.print(MAX_SLOTS - total_slots);
        lcd.print(".");
        lcd.setCursor(0, 1);
        lcd.print("                ");
      }
    
    delay(2000);
    servo.write(110); // coboara bariera

    if (total_slots == 5) {
      if (leave == 1) {
        PORTB |= (1 << PB1); // digitalWrite(LED_SLOT5, HIGH);
        if ((PINB & (1 << PB5)) == LOW) {
          PORTB |= (1 << PB5); // digitalWrite(LED_SLOT1, HIGH);
        }
      }
    }

    if (total_slots == 4) {
      if (come == 1) {
        PORTB &= ~(1 << PB5); // digitalWrite(LED_SLOT1, LOW);
      }
      if (leave == 1) {
        PORTB |= (1 << PB2); // digitalWrite(LED_SLOT4, HIGH);
        if ((PINB & (1 << PB4)) == LOW) {
          PORTB |= (1 << PB4); // digitalWrite(LED_SLOT2, HIGH);
        }
      }
    }

    if (total_slots == 3) {
      if (come == 1) {
        PORTB &= ~(1 << PB4); // digitalWrite(LED_SLOT2, LOW);
      }
      if (leave == 1) {
        if ((PINB & (1 << PB3)) == LOW) {
          PORTB |= (1 << PB3); // digitalWrite(LED_SLOT3, HIGH);
        }
      }
    }

    if (total_slots == 2) {
      if (come == 1) {
        PORTB &= ~(1 << PB3); // digitalWrite(LED_SLOT3, LOW);
      }
      if (leave == 1) {
        PORTB |= (1 << PB4); // digitalWrite(LED_SLOT2, HIGH);
      }
    }

    if (total_slots == 1) {
      if (come == 1) {
        PORTB &= ~(1 << PB2); // digitalWrite(LED_SLOT4, LOW);
      }
      if (leave == 1) {
        PORTB |= (1 << PB5); // digitalWrite(LED_SLOT1, HIGH);
      }
    }

    if (total_slots == 0) {
      if (come == 1) {
        PORTB &= ~(1 << PB1); // digitalWrite(LED_SLOT5, LOW);
      }
    }

    my_int0 = false;
    my_int1 = false;
    come = 0;
    leave = 0;
    just_entered = 0;
    just_exited = 0;
  }

  if (total_slots == 0) { // parcarea este plina
    PORTD |= (1 << PD7); // digitalWrite(LED_SEM_RED, HIGH); -> semafor rosu
    PORTD &= ~(1 << PD6); // digitalWrite(LED_SEM_GREEN, LOW);
  }

  lcd.setCursor(0, 0);
  lcd.print("  Hello, world! ");
  lcd.setCursor(0, 1);
  lcd.print("Locuri libere: ");
  lcd.print(total_slots);
}
